#include <stdio.h>
#include <stdlib.h>
#include "stablo.h"

/*****BIBLIOTEKA, STRUKUTRA SE NALAZI U stablo.h
typedef struct _cvor
{
	int vrednost;
	struct _cvor *levo;
	struct _cvor *desno;
}cvor;
***********************************************/

void greska(char *s)
{
	printf("greska: %s\n", s);
	exit(EXIT_FAILURE);
}

cvor* kreiraj_cvor(int v)
{
	cvor *novi = malloc(sizeof(cvor));
	if(novi == NULL)
		greska("alokacija");
	novi->vrednost = v;
	novi->levo = NULL;
	novi->desno = NULL;
	return novi;
}

cvor* dodaj_u_stablo(cvor *koren, int v)
{
	if(koren == NULL)
		return kreiraj_cvor(v);
	
	if(v < koren->vrednost)
		koren->levo = dodaj_u_stablo(koren->levo, v);
	else
		koren->desno = dodaj_u_stablo(koren->desno, v);
	
	return koren;
}

cvor* ucitaj_stablo()
{
	cvor *s = NULL;
	int x;
	while(1)
	{
		scanf("%d", &x);
		if(x == 0)
			break;
		s = dodaj_u_stablo(s, x);
	}
	return s;
}

cvor* oslobodi_stablo(cvor *koren)
{
	if(koren == NULL)
		return NULL;
	
	oslobodi_stablo(koren->levo);
	oslobodi_stablo(koren->desno);
	free(koren);
	return NULL;
}

void ispis_LKD(cvor *koren)
{
	if(koren == NULL)
		return;
	ispis_LKD(koren->levo);
	printf("%d ", koren->vrednost);
	ispis_LKD(koren->desno);
}

void ispis_KLD(cvor *koren)
{
	if(koren == NULL)
		return;
	printf("%d ", koren->vrednost);
	ispis_KLD(koren->levo);
	ispis_KLD(koren->desno);
}

void ispis_LDK(cvor *koren)
{
	if(koren == NULL)
		return;
	ispis_LDK(koren->levo);
	ispis_LDK(koren->desno);
	printf("%d ", koren->vrednost);
}

int pretraga_stabla(cvor *koren, int v)
{
	if(koren == NULL)
		return 0;
		
	if(koren->vrednost == v)
		return 1;
	
	if(v < koren->vrednost)
		return pretraga_stabla(koren->levo, v);
	else
		return pretraga_stabla(koren->desno, v);
}

int broj_cvorova(cvor *stablo)
{
	if(stablo == NULL)
		return 0;
	
	return 1 + 
		broj_cvorova(stablo->levo) + 
		broj_cvorova(stablo->desno);
}

int broj_listova(cvor *stablo)
{
	//da li je prazno
	if(stablo == NULL)
		return 0;
	
	//da li je list
	if(stablo->levo == NULL && stablo->desno == NULL)
		return 1;
	
	return 
		broj_listova(stablo->levo) + 
		broj_listova(stablo->desno);
}

int suma_cvorova(cvor *stablo)
{
	if(stablo == NULL)
		return 0;
	
	return stablo->vrednost +
		suma_cvorova(stablo->levo) +
		suma_cvorova(stablo->desno);
}

int suma_listova(cvor *stablo)
{
	//da li je prazno
	if(stablo == NULL)
		return 0;
	
	//da li je list
	if(stablo->levo == NULL && stablo->desno == NULL)
		return stablo->vrednost;
	
	return 
		suma_listova(stablo->levo) + 
		suma_listova(stablo->desno);
}

void ispis_listova(cvor *stablo)
{
	if(stablo == NULL)
		return;
	
	if(stablo->levo == NULL && stablo->desno == NULL)
	{
		printf("%d ", stablo->vrednost);
		return;
	}
	
	ispis_listova(stablo->levo);
	ispis_listova(stablo->desno);
}

int dubina(cvor *stablo)
{
	if(stablo == NULL)
		return 0;
	
	int dl = dubina(stablo->levo);
	int dd = dubina(stablo->desno);
	
	return dd > dl ? dd + 1 : dl + 1;
}

int max_uredjeno_stablo(cvor *stablo)
{
	if(stablo == NULL)
		return 0;
	
	while(stablo->desno != NULL)
		stablo = stablo->desno;
	
	return stablo->vrednost;
}

int max_neuredjeno_stablo(cvor *stablo)
{
	if(stablo == NULL)
		return 0;
	
	int maxl = max_neuredjeno_stablo(stablo->levo);
	int maxd = max_neuredjeno_stablo(stablo->desno);
	
	int max_naslednik = maxl > maxd ? maxl : maxd;

	return max_naslednik > stablo->vrednost ?
		max_naslednik :
		stablo->vrednost;
}

//PRAVIMO BIBLIOTEKU PA KOMENTARISEMO MAIN
/*****************************
int main()
{
	cvor *stablo = ucitaj_stablo();
	
	printf("levo - koren - desno\n");
	ispis_LKD(stablo);
	printf("\n");
******************************/
	
	//primeri ispisa:
	//printf("koren - levo - desno\n");
	//ispis_KLD(stablo);
	//printf("\n");
	
	//printf("levo - desno - koren\n");
	//ispis_LDK(stablo);
	//printf("\n");
	
	/*****PRETRAGA**********
	int x;
	printf("unesite vrednost za pretragu: ");
	scanf("%d", &x);

	if(pretraga_stabla(stablo, x))
		printf("postoji element\n");
	else
		printf("ne postoji element\n");
	************************/
	
/******************
	printf("broj cvorova: %d\n", broj_cvorova(stablo));
	printf("broj listova: %d\n", broj_listova(stablo));
	
	printf("suma cvorova: %d\n", suma_cvorova(stablo));
	printf("suma listova: %d\n", suma_listova(stablo));
	
	printf("Listovi:\n");
	ispis_listova(stablo);
	printf("\n");
	
	printf("dubina: %d\n", dubina(stablo));
	
	printf("max. uredjeno: %d\n", max_uredjeno_stablo(stablo));
	printf("max. neuredjeno: %d\n", 
		max_neuredjeno_stablo(stablo));
	
	stablo = oslobodi_stablo(stablo);
}
*****************/